-- Put.applescript
-- Put

--  Created by Nico Rohrbach on 21.04.06.
--  Copyright 2006 Superpiuxel, Nico Rohrbach. All rights reserved.


property default_delimiter : missing value
property warn_image : missing value
property success_image : missing value

property which_panel : missing value
property open_panel_directory : "~/Desktop/"
property file_path : missing value
property proxy_image : missing value
property upload_ready : false

property stored_hostpath : ""
property stored_username : ""
property stored_password : ""




(* ==== EVENT HANDLERS ==== *)


on awake from nib theObject
	
	-- Make the new toolbar, giving it a unique identifier
	set mainToolbar to make new toolbar at end with properties 
		{name:"main toolbar", identifier:"main toolbar identifier", allows customization:true, auto sizes cells:true, display mode:default display mode, size mode:default size mode}
	
	-- Setup the allowed and default identifiers.
	set allowed identifiers of mainToolbar to 
		{"upload identifier", "destination identifier", "customize toolbar item identifer", "flexible space item identifer", "space item identifier", "separator item identifier"}
	set default identifiers of mainToolbar to 
		{"upload identifier", "destination identifier"}
	
	--set selectable identifiers of documentToolbar to {}
	
	-- Create the toolbar items, adding them to the toolbar.
	make new toolbar item at end of toolbar items of mainToolbar with properties 
		{identifier:"upload identifier", name:"upload", label:"Upload", palette label:"Upload file", tool tip:"Upload file", image name:"upload"}
	
	-- Create the toolbar items, adding them to the toolbar.
	make new toolbar item at end of toolbar items of mainToolbar with properties 
		{identifier:"destination identifier", name:"destination", label:"Destination", palette label:"Upload Destination", tool tip:"Set upload destination", image name:"destination"}
	
	-- Assign our toolbar to the window
	set toolbar of theObject to mainToolbar
	
end awake from nib


on will finish launching theObject
	my registerSettings()
	my readSettings()
	--
	set default_delimiter to AppleScript's text item delimiters
	set warn_image to load image "caution"
	set success_image to load image "complete"
	tell text field "filename" of window "main" to register drag types {"file names"}
end will finish launching


on launched theObject
	show window "main"
	my updateUI() -- we update the UI to show if destination server is set
end launched


on clicked theObject
	
	if name of theObject is "choose" then
		my show_open_panel()
		
	else if name of theObject is "dest_status_button" then
		my showDestinationPanel()
		
	else if name of theObject is "dest_cancel" then
		close panel window "destination" with result 0
		
	else if name of theObject is "dest_ok" then
		tell window "destination"
			set new_hostpath to content of text field "url"
			set new_username to content of text field "username"
			set new_password to content of text field "password"
		end tell
		set new_check to {new_hostpath, new_username, new_password}
		if "" is in new_check then
			beep
			tell window "destination"
				set image of image view "dest_status_icon" to warn_image
				set content of text field "dest_status" to "You must fill all fields!"
			end tell
		else
			close panel window "destination" with result 1
		end if
		
	end if
end clicked


-- This event handler is called when the user clicks on one of the toolbar items
on clicked toolbar item theObject
	
	if identifier of theObject is "upload identifier" then
		my uploadJob()
		
		
	else if identifier of theObject is "destination identifier" then
		my showDestinationPanel()
		
	end if
end clicked toolbar item


on drop theObject drag info dragInfo
	-- Get the list of data types on the pasteboard
	set dataTypes to types of pasteboard of dragInfo
	
	-- We are only interested in "file names"
	if "file names" is in dataTypes then
		
		-- Initialize the list of files to an empty list
		set theFiles to {}
		
		-- We want the data as a list of file names, so set the preferred type to "file names"
		set preferred type of pasteboard of dragInfo to "file names"
		
		-- Get the list of files from the pasteboard
		set theFiles to contents of pasteboard of dragInfo
		
		-- Make sure we have only one item
		if (count of theFiles) > 1 then
			beep
			display alert "You droped more than one file." as informational message 
				"Resize can process files only individually." default button "OK" attached to window "main"
			set preferred type of pasteboard of dragInfo to ""
			set file_path to missing value
			my updateUI()
			return false
			
		else if (count of theFiles) > 0 then
			try
				set file_path to first item of theFiles
				
				set temp_filename to fileNamefromPath(file_path)
				
				if websaveString(temp_filename) is false then
					beep
					display alert "The droped file have no websave name." as informational message 
						"The filename can only contain the characters A to Z (uppercase and lowercase), " & 
						"The numbers from 0 to 9 and the characters (- _ .)." default button "OK" attached to window "main"
					set preferred type of pasteboard of dragInfo to ""
					set file_path to missing value
					my updateUI()
					return false
				end if
				my updateUI()
				return true
				
			on error err_msg
				set preferred type of pasteboard of dragInfo to ""
				set file_path to missing value
				my updateUI()
				return false
			end try
		end if
	end if
end drop


on panel ended theObject with result withResult
	
	if which_panel is equal to "open_panel" then
		if withResult is 1 then -- 1 is returned if the user NOT clicks cancel
			-- Convert the list into a list of strings separated by return characters that we can put in the 'path names' text view
			-- For some unknown (as of yet) you must coerce the 'path names' to a list (even though it is defined as list).
			set open_panel_path to (path names of open panel as list)
			set AppleScript's text item delimiters to return
			set file_path to open_panel_path as string
			set AppleScript's text item delimiters to default_delimiter
			
			-- check for websave filename			
			set temp_filename to fileNamefromPath(file_path)
			
			if websaveString(temp_filename) is false then
				beep
				display alert "This file have no websave name." as informational message 
					"The filename can only contain the characters A to Z (uppercase and lowercase), " & 
					"The numbers from 0 to 9 and the characters (- _ .)." default button "OK"
				set file_path to missing value
				my updateUI()
				return false
			end if
			
			my updateUI()
		end if
		
	else if which_panel is equal to "destination_panel" then
		if withResult is 1 then -- 1 is returned if the user NOT clicks cancel
			tell window "destination"
				set stored_hostpath to content of text field "url"
				set stored_username to content of text field "username"
				set stored_password to content of text field "password"
			end tell
		end if
		
		my updateUI()
	end if
end panel ended


on should quit after last window closed theObject
	return true
end should quit after last window closed


on will quit theObject
	my writeSettings()
end will quit


on choose menu item theObject
	
	if name of theObject is "menu_open" then
		my show_open_panel()
		
	else if name of theObject is "menu_upload" then
		my uploadJob()
		
	end if
end choose menu item



(* ==== PRIVATE HANDLERS ==== *)

on uploadJob()
	-- check for file
	if upload_ready is false then
		beep
		display alert "No file for upload is choosen." as informational message 
			"Choose a file for upload and try again." default button "OK" attached to window "main"
		return
	end if
	
	-- check upload settings
	set settings_checksum to stored_hostpath & stored_username & stored_password as list
	if settings_checksum contains "" then
		beep
		display alert "Wrong destination settings." as informational message 
			"Make sure you have set the remote file location, username and password correctly." default button "OK" attached to window "main"
		return
	end if
	
	(* UPLOAD THE FILE *)
	
	-- prepare the panel
	set uses threaded animation of progress indicator "wheel" of window "status" to true
	set indeterminate of progress indicator "wheel" of window "status" to true
	set content of text field "statusinfo" of window "status" to ("Upload \"" & fileNamefromPath(file_path) as text) & "\""
	
	-- display the panel
	display panel window "status" attached to window "main"
	tell progress indicator "wheel" of window "status" to start
	
	set temp to put_action(file_path, stored_hostpath, stored_username, stored_password) -- local_file, remote_url, the_username, the_password
	if first item of temp is false then -- error occured
		tell progress indicator "wheel" of window "status" to stop
		beep
		close panel window "status"
		if (last item of temp as text) is "-128" then -- user has aborted
			return
		else
			display alert "Error during upload!" as critical message (second item of temp as text) default button "OK" attached to window "main"
			return
		end if
	end if
	
	tell progress indicator "wheel" of window "status" to stop
	close panel window "status"
	
	-- set success path before we clear file_path
	set success_path to file_path
	
	-- clear UI
	set file_path to missing value
	my updateUI()
	
	-- show the success in glass bar
	set contents of text field "result_text" of window "main" to ("Upload of \"" & fileNamefromPath(success_path) as text) & "\" successfull"
	set image of image view "result_icon" of window "main" to success_image
	
end uploadJob


on put_action(local_file, remote_url, the_username, the_password) -- main put handler
	-- curl -T uploadfile -u user:passwd ftp://ftp.upload.com/
	with timeout of 300 seconds -- 5 min
		try
			set put_result to do shell script "/usr/bin/curl -T" & 
				space & quoted form of local_file & 
				space & "-u" & space & the_username & ":" & the_password & 
				space & remote_url
		on error err_msg number err_num
			return {false, err_msg, err_num}
		end try
		return {true, put_result}
	end timeout
end put_action


on fileNamefromPath(temp_file_path)
	set AppleScript's text item delimiters to "/"
	set temp_file_name to last text item of temp_file_path
	set AppleScript's text item delimiters to default_delimiter
	return temp_file_name
end fileNamefromPath


on megabyteString(size_in_bytes)
	set size_in_megbytes to size_in_bytes / ((1024) ^ 2)
	set size_string to (((round (10 * size_in_megbytes)) / 10) as text)
	if size_string is equal to "0.0" then
		set size_string to "Size: Less than 100 KB"
	else
		set size_string to "Size: " & size_string & " MB"
	end if
	return size_string
end megabyteString


on show_open_panel()
	-- preparing open panel
	tell open panel
		set title to "Choose file"
		set prompt to "Choose"
		set treat packages as directories to false
		set can choose directories to false
		set can choose files to true
		set allows multiple selection to false
	end tell
	-- show the panel
	set which_panel to "open_panel"
	display open panel in directory open_panel_directory attached to window "main"
end show_open_panel


on showDestinationPanel()
	tell window "destination"
		try
			delete image of image view "dest_status_icon"
		end try
		set content of text field "dest_status" to ""
		--
		set content of text field "url" to stored_hostpath
		set content of text field "username" to stored_username
		set content of text field "password" to stored_password
		set first responder to text field "url"
	end tell
	set which_panel to "destination_panel"
	display panel window "destination" attached to window "main"
end showDestinationPanel


on updateUI()
	
	-- is destination set complete?
	set new_check to {stored_hostpath, stored_username, stored_password}
	if "" is in new_check then
		-- NOT complete
		set title of button "dest_status_button" of window "main" to " Destination is not set."
	else
		-- complete
		set title of button "dest_status_button" of window "main" to space & stored_hostpath
	end if
	
	if file_path is not missing value then
		set contents of text field "filename" of window "main" to (file_path as string)
		set size_in_bytes to size of (info for POSIX file file_path) as integer
		set contents of text field "size" of window "main" to megabyteString(size_in_bytes)
		
		set AppleScript's text item delimiters to "."
		set the_suffix to last text item of file_path
		set AppleScript's text item delimiters to default_delimiter
		
		set upload_ready to true
		
		if the_suffix is in {"zip", "sit", "gz"} then
			my loadProxy("archive")
			
		else if the_suffix is in {"dmg", "iso", "sparseimage"} then
			my loadProxy("diskimage")
			
		else if the_suffix is in {"xls"} then
			my loadProxy("excel")
			
		else if the_suffix is in {"png", "tiff", "tif", "jpeg", "jpg", "pict", "gif", "psd", "psb"} then
			my loadProxy("image")
			
		else if the_suffix is in {"mov", "mp4", "m4v", "avi", "mpg", "mpeg"} then
			my loadProxy("movie")
			
		else if the_suffix is in {"mp3", "aac", "aif", "aiff", "wav", "snd"} then
			my loadProxy("music")
			
		else if the_suffix is in {"pdf"} then
			my loadProxy("pdf")
			
		else if the_suffix is in {"ppt"} then
			my loadProxy("powerpoint")
			
		else if the_suffix is in {"doc"} then
			my loadProxy("word")
			
		else
			my loadProxy("default")
			
		end if
	else
		set upload_ready to false
		my loadProxy("dottedIcon")
		set contents of text field "filename" of window "main" to ""
		set contents of text field "size" of window "main" to " "
	end if
	
	set contents of text field "result_text" of window "main" to ""
	-- set image of image view "result_icon" of window "main" to success_image
	delete image of image view "result_icon" of window "main"
	
end updateUI


on loadProxy(proxy_name)
	-- first try to delete old proxy image,
	-- we don't want a memory leak!
	try
		delete proxy_image
	end try
	-- load the new image
	set proxy_image to load image proxy_name
	set image of image view "proxy" of window "main" to proxy_image
end loadProxy


on websaveString(the_string)
	set uppercase_chars to {"A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z"}
	set lowercase_chars to {"a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z"}
	set number_chars to {"1", "2", "3", "4", "5", "6", "7", "8", "9", "0"}
	set special_chars to {".", "-", "_"}
	set valid_chars to uppercase_chars & lowercase_chars & number_chars & special_chars
	set test_string to every character of the_string
	repeat with this_item in test_string
		if this_item is not in valid_chars then
			return false
		end if
	end repeat
	return the_string
end websaveString


(* Registers the settings (application preferences) with the 'user defaults'. *)
on registerSettings()
	tell user defaults
		-- Add all of the new default entries
		make new default entry at end of default entries with properties {name:"hostpath", contents:stored_hostpath}
		make new default entry at end of default entries with properties {name:"username", contents:stored_username}
		make new default entry at end of default entries with properties {name:"password", contents:stored_password}
		
		-- Now we need to register the new entries in the user defaults
		register
	end tell
end registerSettings


(* Reads the settings (application preferences) from the 'user defaults'. *)
on readSettings()
	tell user defaults
		set stored_hostpath to contents of default entry "hostpath"
		set stored_username to contents of default entry "username"
		set stored_password to contents of default entry "password"
	end tell
end readSettings


(* Writes the settings (application preferences) to the 'user defaults'. *)
on writeSettings()
	tell user defaults
		set contents of default entry "hostpath" to stored_hostpath
		set contents of default entry "username" to stored_username
		set contents of default entry "password" to stored_password
	end tell
end writeSettings
